/** 
 *  @file   IFrameQBufMgr.xdc
 *
 *  @brief      Interface file for FrameQbufMgr.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */



package ti.syslink.ipc.rtos.interfaces;


import xdc.runtime.IGateProvider;
import ti.sdo.utils.NameServer;
import ti.sdo.utils.MultiProc ;


interface IFrameQBufMgr inherits IFrame {

    /*! type used for API return code. */
    typedef UInt32 Status ;

   /*! @_nodoc
     *  Delegate writer needs...
     */
    const UInt32 CREATED = 0x05251995;
    const UInt32 VERSION = 1;

    /*! Type of creation */
    const UInt32 STATIC_CREATE                 = 0x1;
    /*! Type of creation */
    const UInt32 STATIC_CREATE_USEDREGION      = 0x2;
    /*! Type of creation */
    const UInt32 DYNAMIC_CREATE                = 0x4;
    /*! Type of creation */
    const UInt32 DYNAMIC_CREATE_USEDREGION     = 0x8;
    /*! Type of open */
    const UInt32 STATIC_OPEN                   = 0x10;
    /*! Type of open */
    const UInt32 DYNAMIC_OPEN                  = 0x20;
    /*! Type of open */
    const UInt32 INVALID                       = 0xFF;

    /*!
     *============CACHE_LINESIZE ================
     * Cache line size.
     *
     */
    config  UInt32 CACHE_LINESIZE = 128 ;

    /*!
     *  Maximum length of the  name strings .
     */
    config UInt32  MAX_NAMELEN = 32 ;
    /*!
     *  Maximum number of instances that can be created for this module .
     */
    config UInt32  MAXINSTANCES = 64 ;

    /*!
     *  Maximum number of Free Frame pools pools managed by FrameQBufMgr
     *  instance.
     */
    const UInt32 MAX_POOLS = 64 ;
    /*!
     *  Maximum number of static free Frames  managed by FrameQBufMgr
     *  instance for a Free Frame pool.
     */
     const UInt32   POOL_MAXFRAMES = 32;
    /*!
     *  Maximum number of additional static free headers  managed by FrameQBufMgr
     *  instance for each Free Frame pool.
     */
     const UInt32   POOL_MAXHDRS  = 64;

    /*!
     *  Maximum number of frame buffers in a frame managed by in
     *  FrameQBufMgr instance.
     */
    const UInt32 MAX_FRAMEBUFS = 8 ;

    /*!
     *  Maximum number of dynbuffers managed by each buffer pool in
     *  FrameQBufMgr instance.
     */
    const UInt32 POOL_DYNAMIC_MAXFRAMES = 1;

    /*!
     *  Maximum number of additional dynamic free headers  managed by FrameQBufMgr
     *  instance for each Free Frame pool.
     */
     const UInt32   POOL_DYNAMIC_MAXHDRS  = 1;

    /*! @_nodoc
     *   Denotes header buffer is not pointing to any  static frame or dynamicframe.
     */
     const UInt8 NONE_FRAME   = 0;

    /*!
     *   Denotes Frame belongs to static frames that are populated at create time of the instance.
     */
     const UInt8 STATIC_FRAME = 1;

    /*!
     *   Denotes Frame belongs to dynamic frame grougp that are added dynamically at runtime.
     */
     const UInt8 DYNAMIC_FRAME = 2;

   /*!
     *  Event Number to be used by  FrameQBufMgr module.
     */
    config UInt32 NOTIFY_RESERVED_EVENTNO  =  3 ;

    /*!
     *  If set  to true FrameQBufMgr  instances will use dafault gate set in
     *  FrameQBufMgr.
     *  If default gate is not set in FrameQBufMgr, it uses GateMP's default 
	 *  gate. Set it to zero if   FrameQBufMgr needs to  create  gate for a
     *  instance (remoteProtect and localProtect  flags should be valid in
     *  params in this case).
     */
	config Bool usedefaultgate   = false;

    /*!
     *  ======== Interface ========
     *  Enumeration of FrameQBufMgr interface types.Denotes different implementation types.
     *
     *  @value(INTERFACE_SHAREDMEM) FrameQNufMgr implementation (FrameQBufMgr_ShMem Implementation) on shared memory.
     *
     */

    enum Interface{
        INTERFACE_SHAREDMEM = 0x0,
        INTERFACE_TILERMEM  = 0x1,
        INTERFACE_NONE      = 0x2
    };

    /*!
     *  ======== AddrType ========
     *  Enumerations to indicate address types used for translation.
     *
     *  @value(AddrType_Virtual) Virtual address on calling process on DSP where
     *                           MMU is not configured it could be the physical
     *                           address.
     *  @value(AddrType_Portable)This is the shared region address incase
     *                           sharedmemory manager plugged in to
     *                           frameQbufMgr. In case of tiler it could be the
     *                           tiler specific portable address
     *  @value(AddrType_EndValue)End delimiter indicating start of invalid
     *                           values for this enum
     */
    enum AddrType{
    AddrType_Virtual = 0u,
    AddrType_Portable = 1u,
    AddrType_EndValue = 2u
    };

   /*!
     *  ======== BufType ========
     *  Enumerations to indicate address types used for translation
     *
     *  @value(BUF_FRAMEHEADER) Denotes buffer is  of type used for frame headers.
     *
     *  @value(BUF_FRAMEBUF)  Denotes buffer is  of type used for frame buffers
     *
     *  @value(AddrType_EndValue)End delimiter indicating start of invalid
     *                           values for this enum
     *
     */
    enum BufType {
        BUF_FRAMEHEADER = 0u,
        BUF_FRAMEBUF = 1u,
        BUF_ENDVALUE = 2u
    } ;

    /*!
     *  ======== CpuAccessFlags ========
     *  Enumeration of CPU access flags for frame buffers. For frame headers CPUACCESS is assumed to be TRUE.
     *
     *  @value(FRAMEBUF0_CPUACCESS) If specified first frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF1_CPUACCESS) If specified second frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF2_CPUACCESS) If specified third frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF3_CPUACCESS) If specified fourth frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF4_CPUACCESS) If specified fifth frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF5_CPUACCESS) If specified sixth frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF6_CPUACCESS) If specified seventh frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF7_CPUACCESS) If specified eigth frame buffer is accessed  through CPU( direct memory access).
     *
     */
    enum CpuAccessFlags{
        FRAMEBUF0_CPUACCESS    =  0x10000,
        FRAMEBUF1_CPUACCESS    =  0x20000,
        FRAMEBUF2_CPUACCESS    =  0x40000,
        FRAMEBUF3_CPUACCESS    =  0x80000,
        FRAMEBUF4_CPUACCESS    = 0x100000,
        FRAMEBUF5_CPUACCESS    = 0x200000,
        FRAMEBUF6_CPUACCESS    = 0x400000,
        FRAMEBUF7_CPUACCESS    = 0x800000
    };

   const UInt32 FRAMEBUF_CPUACCESSFLAGS_BITOFFSET = 16;
    /*!
     *  ========  NotifyFunc ========
     *  Prototype of the FrameQbufMgr call back function.
     *
     *
     */
     typedef void (* NotifyFunc) ( Handle , Ptr , UInt16) ;


    /*!
     *  ======== Config ========
     *  mouduleconfig structure.
     *
     *  @field(sharedAddr)    Shared region address for module initialization.
     *
     *  @field(sharedAddrSize) Size of the shared region .
     *  @field (cacheFlags)    Cache flags to perform cache operations on module wide shared control structure.
     *
     *  @field(eventNo)        The event number used for notification.
     *  @field(name)           Name for the module.
     *  @field(shCtrlOpenFlag) Boolean flag if set to FALSE, populate the shared control info
     *                         required between  processors for this module. If it set to TRUE,
     *                         module initialize call opens the shared control info.Ideally master processor
     *                         set this flag to FALSE to crate the shared control info.
     */

    struct Config
    {

        UInt32 eventNo;
        Bool   useNameServer;
        Bool   maxInstances;
    } ;

    /*!
     *  ======== OpenParams ========
     *  Common paramters requiredto open a FrameQBufMgr instance of any implementation.
     *
     *  @field(name)           Name of the instance to open.
     *
     *  @field (cacheFlags)    Cache flags to perform cache operations .
     *
     *  @field(sharedAddr)     Shared addr in case open by address is required.
     *  @field(gate)           gate for protection.
     */
    struct CommonOpenParams {
        String                name;
        UInt32                cpuAccessFlags;
        Ptr                   sharedAddr;
       /* TBD:  */
        IGateProvider.Handle  gate;
       /*TBD */
    };

    /*!
     *============Attrs_STRUCT_PADDING ================
     * Padding required for the Attrs structure.
     *
     */
    config UInt32  Attrs_STRUCT_PADDING = ((  CACHE_LINESIZE - (  (4 * 8) + (MAX_NAMELEN)+ 2 )) /2) ;

    /*!
     *  ======== Attrs ========
     *  Shared  control strucrue for the instance attributes.
     *
     *  @field(status)  Flag  denoting whether instance is created or not.
     *  @field(version) Version of the  instance.
     *  @field(localProtect)   Local protection type.See {@link #GateMP.LocalProtect}
     *  @field(remoteProtect)  Remote protection types.See {@link #GateMP.RemoteProtect}
	 *  @field(gateMPAddr) GateMP address (shm safe)
     *  @field(name)    Name of the instance.
     *
     *  @field(sharedDataBufAddr) Holds Data buffer start address provided at  instance crate time.
     *
     *  @field(sharedAddrSize)    The size of the data buffer.
     *
     *  @filed(numFreeFramePools)  Number of free frame pools maintained by this instance.
     *  @filed(entryId)  Unique id of the instance in the system. Combination of
     *                   creator procId and instance no on that processor.
     *
     */
    struct Attrs {
        volatile UInt32      status ;  /* boolean flag  denotes whether it is created or not
                               */
        volatile UInt32      version ; /* version info */
        volatile UInt32      localProtect;
        volatile UInt32      remoteProtect;
		volatile Ptr         gateMPAddr; /* GateMP address (shm safe)     */

        volatile char        name[MAX_NAMELEN] ;
        volatile Ptr         sharedDataBufAddr ; /* Address of the top of the big memory  that is split
                                         * into   datablocks( data buffers)
                                         */
        volatile UInt32      sharedAddrSize ; /* Size of the memory pointed to by sharedDataBufAddr
                                      */
        volatile UInt32      numFreeFramePools ; /* Number of bufpools in use */
		volatile UInt32      numNotifyEntries;

        volatile UInt16      entryId;

    }

    /*!
     *  ======== FreeFramePoolObj ========
     *  Free Frame pool structure to keep track of the free frames in a frame pool
     *
     *
     */
    struct FreeFramePoolObj {
        volatile UInt32 numFrms;
        volatile UInt32 freeFrms;
        volatile UInt32 numFrmHdrBufs;
        volatile UInt32 freeFrmHdrBufs;
        volatile UInt32 frmHdrBufSize;
        volatile UInt32 numBufsInFrame;
        volatile UInt32 numDynamicHdrBufs;
        volatile UInt32 freeDynamicHdrBufs;
        volatile UInt32 numDynamicFrames;
        volatile UInt32 freeDynamicFrames;
        volatile UInt32 frmStartAddrp;
        volatile UInt32 frmEndAddrp;
        volatile UInt32 hdrBufStartAddrp;
        volatile UInt32 hdrBufEndAddrp;
        volatile UInt32 frmBufsStartAddrp;
        volatile UInt32 frmBufsEndAddrp;
	    volatile UInt32 frmBufSize[MAX_FRAMEBUFS];
		volatile UInt32 frmRefCnt[POOL_MAXFRAMES];
	    volatile UInt32 dynFrame[POOL_DYNAMIC_MAXFRAMES];
	    volatile UInt32 dynFrmRefCnt[POOL_DYNAMIC_MAXFRAMES];
	    volatile UInt32 hdrBuf_refCntIndex[POOL_MAXHDRS];
	    volatile UInt32 dynHdrBuf_refCntIndex[POOL_DYNAMIC_MAXHDRS];
	    volatile UInt32 dynHdrBufs[POOL_DYNAMIC_MAXHDRS];
	    volatile Bool   hdrBufIsUsed[POOL_MAXHDRS];
	    volatile Bool   dynHdrBufIsUsed[POOL_DYNAMIC_MAXHDRS];
	    volatile UInt8  hdrBuf_refCntIndexType[POOL_MAXHDRS];
	    volatile UInt8  dynHdrBuf_refCntIndexType[POOL_DYNAMIC_MAXHDRS];
    }

    /*!
     *  ======== ControlStruct ========
     *  Shared control structure for the FrameQBufMgr instance
     *
     *  @field(attrs)        Shared control info of type {@link #Attrs}.
     *  @field(bufPool)      Array of buffer pools of type {@link #bufPoolList}.
     *
     */
    struct ControlStruct {
        Attrs       attrs;
        FreeFramePoolObj freeFramePool[MAX_POOLS] ;
    }

    /*! Initialize the passed pointer with the current parameters
     *
     *  @param(cfgParams)   Pointer to parameter structure to be initialized.
     */
    Int32 getConfig (Config * cfgParams) ;

    /*!
     *  Whether to have this module use the NameServer or not.
     *
     *  Currently defaulted to false because static creation of NameServer
     *  instances is not supported.
     */
    metaonly config Bool useNameServer = true;
    /*!
     *  ======== nameSrvPrms ========
     *  This Params object is used for temporary storage of the
     *  module wide parameters that are for setting the NameServer instance.
     */
    metaonly config NameServer.Params nameSrvPrms;

    /*!
     *  Section name is used to place the names table
     */
    metaonly config String tableSection = null;


   /*!
     *============nameServer ================
     * Handle to the name server.
     *
     */
     config NameServer.Handle  nameServer = null;


    /*!
     *  IGateProvider Proxy
     *
     *  Module used to provide default critical region management when accessing
     *  the shared memory. The default is {@link ti.sdo.ipc.gates.GatePeterson}.
     *
     *  Requires Arm to be procId 0 or 1 and the DSP be 1 or 0.
     */
    proxy GateProxy inherits IGateProvider;

    /*!
     *  ======== getHandle ========
     *  Function to create a nameServer.
     *
     *  @param(id)       Id of the FrameQBufMgr instance.
     *
     *  @b(returns)      Returns instance handle  if it  is created or opened on the caller processor.
     *
     */
     Ptr getHandle (UInt32 id ) ;

    /*!
     *  ========= CreateParams ==========
     *  Structure defining common instance parameters for the FrameQ
     *  module.Should be the first element  in the create params strcture of
     *  the actual implementation.
     *  @field(size)  Size of the Create params structure(Should be the size of the actual
     *                implementation's Create params structure in which this FrameQ_CreateParams is the
     *                first element.
     *  @field(interface) type of implementation. See {@link Interface}.
     *  @field(name)  Name of the instance.
     *  @field(openFlag) Denotes whether create should actually create/open instance.
     */
    struct CreateParams {
        UInt32            size;
        Interface         ctrlInterfaceType;
        String            name;
        Bool              openFlag;
    } ;

    /*!
     *  ========= FrameBufParams ==========
     *  Structure defining common frame buf params for tiled as well as non tiled buffers
     *  @field(size)	size for non tiled buffer.
     *  Structure	defining parameters for the Tiled data buffers
     *  @field(pixelFormat)  pixel format for tiled buffer.
     *  @field(height) height .
     *  @field(width)  width .
     *  @field(stride) stride of container, must be multiple of page size.
     *  @field(align)  Alignment of the buffer .
     */
    struct FrameBufParams {
        UInt32         size;
        UInt32         pixelFormat;
        UInt32         height;
        UInt32         width;
	    UInt32         align;
    } ;

    /*!
     *  ========= NotifyParams ==========
     *  Structure defining notification parameter structure.
     *
     *  @field(notifyType) notification type.See {@link SysLink.NotifyType}.
     *  @field(watermark) Minumum number of free frames required to generate
     *                    notification   Same water mark for all the individual
     *                    freeFramePools in the set.
     *  @field(cbFxnPtr)  Cacll back function.
     *  @field(cbContext) Context pointer that needs to be passed to call back
     *                    function.
     *  @field(cbClientHandle) Handle that needs to be passed as first arg to
     *                         call back.Ideally this should be callers client
     *                         handle.
     */
    struct NotifyParams {
         SysLink.NotifyType  notifyType;
         UInt32              watermark;
         NotifyFunc          cbFxnPtr;
         Ptr                 cbContext;
         Ptr                 cbClientHandle;
    };


instance:

    /*!
     *  ======== alloc ========
     *  Function to allocate a buffer from the FrameQBufMgr instance.
     *
     *  @param(frame)  Location to receive the allocated frame pointer.
     *
     *  @b(returns)       returns 0 if  successfully allocated the buffer.
     *
     */
    Int32 alloc (Frame * frame );

   /*!
     *  ======== allocv ========
     *  Function to allocate  multiple frames from the FrameQBufMgr instance from
     *  the free Frame pool nos given.
     *
     *  @param(framePtr)  array of pointers to receive allocate frame pointers.
     *
     *  @param(freeQId)   array of freeFrame poolNos to allocate frames from.
     *                    if API retruns success, this represents ths actual size of the buffer allocated.
     *  @param(numFrames)      Number of frames to allocate.
     *
     *  @b(returns)       returns value greater than zero
     *                    if  successfully allocated frames.
     *
     */
    Int32 allocv (Frame        framePtr[],
                  UInt32       freeQId[],
                  UInt8        *numFrames);

    /*!
     *  ======== free  ========
     *  Function to free frame
     *
     *  @param(frame)       frame to be freed.
     *
     *  @b(returns)       returns 0 if  successfully freed the  allocated buffer.
     *
     */
    Int32 free (Frame frame);
    /*!
     *  ======== freev  ========
     *  Function to free frames
     *
     *  @param(framePtr)       array of frames to be freed.
     *
     *  @b(returns)       returns 0 if  successfully freed the  allocated frames.
     *
     */
     Int32  freev (Frame  framePtr[], UInt32 numFrames);

    /*!
     *  ======== add ========
     *  Function to add  free frames dynamically  which are allocated out side of
     *  FrmaeQbufMgr through the  same memory manager
     *
     *  @param(freeQId)  freeFrame Pool No to whcih the frame needs to be
     *                   added.
     *
     *  @param(framePtr) frame to be added.
     *
     *  @b(returns)       returns 0 if  successfully added the frame.
     *
     */
    Ptr add (UInt8 freeQId);

    /*!
     *  ======== remove ========
     *  Function to remove  dynamically added frames which are allocated out side of
     *  FrmaeQbufMgr through the  same memory managers that are plugged in to
     *  FrameQBufMgr instance.
     *
     *  @param(freeQId)  freeFrame Pool No to whcih the frame needs to be
     *                   added.
     *
     *  @param(framePtr) frame to be added.
     *
     *  @b(returns)       returns 0 if  successfully added the frame.
     *
     */
    Int32 remove (UInt8 freeQId, Frame framePtr);


    /*!
     *  ======== registerNotifier ========
     *  Function to Register notification with the instance.  It internally registers
     * notification with the individual free queues( free frame pools).
     * If alloc on free queue 0 failed,Notification will be generated if free frames
     * in that queue becomes more than watermark(all the free frames in that queue)
     *
     *  @param(notifyParams)  Pointer to  {@link NotifyParams} structure.
     *
     *  @b(returns)                 returns >0 if  successfully registered the call back function.
     *
     */
    Int32 registerNotifier (NotifyParams *notifyParams);

    /*!
     *  ======== unregisterNotifier ========
     *  Function to unregister the registered call back function with the instance..
     *
     *  @b(returns)   Returns positive value if  successfully unregistered the call back function.
     *
     */
    Int32 unregisterNotifier ();


     /*!
     *  ======== dup ========
     *  API to duplicate the given frame i.e it increments the corresponding
     *  frames 's reference count.It internally allocates  headers for the duped
     *  frames.It fails if it is not able to dup all the frames.
     *
     *  @param(buf)    Pointer to the frame which needs to be duplicated.
     *  @param(size)   Size of the buffer.
     *  @param(dupCount) Duplication count.
     *
     *
     */
     Int32 dup (Frame   framePtr,
	            Frame   dupedFramePtr[],
                UInt32  numDupedFrames);

     /*!
     *  ======== dupv ========
     *  API to duplicate the given frames i.e it increments the corresponding
     *  frames 's reference count.It internally allocates  headers for the duped
     *  frames.It fails if it is not able to dup all the frames.
     *
     *  @param(buf)    Pointer to the frames which needs to be duplicated.
     *  @param(size)   Size of the buffer.
     *  @param(dupCount) Duplication count.
     *
     *
     */
     Int32 dupv (Frame   framePtr[],
                 Frame  **dupedFramePtr,
                 UInt32  numDupedFrames,
                 UInt32  numFrames);
   /*!
     *  ======== getNumFreeFrames ========
     *  Function to get the number of free frames available in the in 
	 *  FrameQbufMgr's FreeFramePool 0. The returned number free frame may not 
	 *  exist in the frameQbufMgr  after this call.This is because 
	 *  if any allocs are done by the other FrameQ writers on the 
	 *  same FrameQBufMgr.If frees are not other frameQ writers the free Frames 
	 *  would be even more than the value that we have got.
     *
     *  @param(numFrames) Outputparameter indicates number of free frames 
	 *                    .
     *
     *  @b(returns)       returns 0 if  successfully returned the number of 
	 *                    free frames.
     *
     */
     Int32 getNumFreeFrames (UInt32* numFreeFrames) ;

    /*!
     *  ======== getvNumFreeFrames ========
     * Function to get the number of free frames available in the in the given 
	 * Free FramePool. 
	 *
     *  @param(numFreeFrames) Outputparameter indicates number of free frames in
	 *                    each given free quque at that instance.
	 *  @param(filledQId) Array of free Queue ids( freeFramePool nos).
	 *  @param(numFreeQids) Number of free queueids same as freeFrame PoolNo of 
	 *                      plugged in FrameQBufMgr .
     *
     *  @b(returns)       returns 0 if  successfully returned the frames.
     *
     */
	Int32 getvNumFreeFrames(UInt32            numFreeFrames[],
                            UInt8             freeFramePoolNo[],
                            UInt8             numFreeFramePools);

    /*!
     *  ======== writeBack ========
     * Write back the contents  of a frame including frame buffers
     */
    Int32 writeBack(Frame frame);

    /*
     *  ======== invalidate ========
     * Invalidate the contents  of a frame including frame buffers.
     */
    Int32 invalidate(Frame frame);

    /*!
     *  ======== writeBackHeaderBuf ========
     * Write back the contents  of buffer.buffer is treated as frame header
     */
    Int32 writeBackHeaderBuf(Ptr headerBuf);

    /*!
     *  ======== invalidateHeaderBuf ========
     * invalidates contents  of buffer . buffer is treated as frame header
     */
    Int32 invalidateHeaderBuf(Ptr headerBuf);

    /*!
     *  ======== writeBackFrameBuf ========
     * Write back the contents  of buffer . buffe is treated as frame  buffer.
     * bufNumInFrame denotes  buffer number in the frame.This is  to identify
     * the cache flags and cpu access flags for the  buffer.
     */

    Int32 writeBackFrameBuf(Ptr frameBuf, UInt32 size, UInt8 bufNumInFrame);

    /*
     *  ======== invalidateFrameBuf ========
     * Invalidate the contents  of frame buffer .
     *
     * bufNumInFrame denotes which buffer it is  to identify the cache flags and
     * cpu access flags for the  buffer. This is the sequence number of frame buffer
     * in the frame.
     */
    Int32 invalidateFrameBuf(Ptr frameBuf, UInt32 size, UInt8 bufNumInFrame);

    /*
     *  ======== translateAddr ========
     * Function to translate address betweem different types.
     * see also {@link AddrType}
     */
    Int32 translateAddr (Ptr *     dstAddr,
                         AddrType  dstAddrType,
                         Ptr       srcAddr,
                         AddrType  srcAddrType,
                         BufType   bufType);
   /*!
     *  ======== getId ========
     *  Function to get the id of the instance
     *
     *  @b(returns)     Returns FrameQBufMgr Id  for the handle.
     */
      UInt32 getId () ;

    /*!
     *  ======== translateAddr ========
     * Function to determine whether cache calls(writeback/invalidate) are 
     * enabled for frame headers.
     */
    bool isCacheEnabledForHeaderBuf();

    /*!
     *  ======== translateAddr ========
     * Function to determine whether cache calls(writeback/invalidate) are 
     * enabled for frame buffers.
     */
    bool isCacheEnabledForFrameBuf( UInt8 framebufIndex);

    /*!
     *  ======== FrameQ_control ========
     *  Provides a hook to perform implementation dependent operation.
     *
     *  @param(cmd) command to exectue.
     *  @param(arg) argument of type void*.
     */
        Int32 control (Int32 cmd, Ptr arg);


}
